@@ -32,7 +32,7 @@
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-
+#include <gdk/gdkx.h>
 
 #ifdef HAVE_LIBNOTIFY
 #include <libnotify/notify.h>
@@ -263,6 +263,11 @@ downloader_view_finalize (GObject *object)
 
 	if (priv->status_icon != NULL)
 	{
+		/* FIXME: this should not be necessary (setting visible to
+		 * FALSE), but we have to work-around a GTK+/gnome-panel bug:
+		 *   http://bugzilla.gnome.org/show_bug.cgi?id=340110
+		 */
+		gtk_status_icon_set_visible (priv->status_icon, FALSE);
 		g_object_unref (priv->status_icon);
 		priv->status_icon = NULL;
 	}
@@ -415,6 +420,39 @@ ephy_download_get_remaining_time (WebKitDownload *download)
 	return remaining_time;
 }
 
+static void
+do_open_downloaded_file (DownloaderView *dv, WebKitDownload *download, gboolean open_location)
+{
+	DownloaderViewPrivate *priv = dv->priv;
+	GdkDisplay *gdk_display;
+	const char *destination_uri = webkit_download_get_destination_uri (download);
+	GFile *downloaded_file = g_file_new_for_uri (destination_uri);
+
+	gdk_display = gtk_widget_get_display (priv->window);
+	if (open_location)
+	{
+		ephy_file_browse_to (downloaded_file, gdk_x11_display_get_user_time (gdk_display));
+	}
+	else
+	{
+		ephy_file_launch_handler (NULL, downloaded_file,
+					  gdk_x11_display_get_user_time (gdk_display));
+	}
+	g_object_unref (downloaded_file);
+}
+
+static void
+open_downloaded_file (DownloaderView *dv, WebKitDownload *download)
+{
+	do_open_downloaded_file (dv, download, FALSE);
+}
+
+static void
+open_downloaded_file_location (DownloaderView *dv, WebKitDownload *download)
+{
+	do_open_downloaded_file (dv, download, TRUE);
+}
+
 static void
 update_download_row (DownloaderView *dv, WebKitDownload *download)
 {
@@ -427,7 +465,7 @@ update_download_row (DownloaderView *dv, WebKitDownload *download)
 	char *remaining, *file, *cur_progress, *name;
 	struct tm;
 	int percent = 0;
-
+	DownloadAction action;
 #ifdef HAVE_LIBNOTIFY
 	char *downloaded;
 #endif
@@ -451,6 +489,22 @@ update_download_row (DownloaderView *dv, WebKitDownload *download)
 		downloader_view_remove_download (dv, download);
 		return;
 	case WEBKIT_DOWNLOAD_STATUS_FINISHED:
+		action = GPOINTER_TO_INT(g_object_get_data (G_OBJECT(download),
+							    "download-action"));
+
+		switch (action)
+		{
+		case DOWNLOAD_ACTION_OPEN:
+			open_downloaded_file (dv, download);
+			break;
+		case DOWNLOAD_ACTION_OPEN_LOCATION:
+			open_downloaded_file_location (dv, download);
+			break;
+		default:
+			g_assert_not_reached ();
+			break;
+		}
+
 		downloader_view_remove_download (dv, download);
 #ifdef HAVE_LIBNOTIFY
 		downloaded = g_strdup_printf (_("The file “%s” has been downloaded."), 
@@ -616,6 +670,11 @@ downloader_view_add_download (DownloaderView *dv,
 	char *downloading;
 #endif
 
+	/* dv may be unrefed inside update_download_row if the file
+	 * downloaded completely while the user was choosing where to
+	 * put it, so we need to protect it
+	 */
+	g_object_ref (dv);
 	g_object_ref (download);
 
 	gtk_list_store_append (GTK_LIST_STORE (dv->priv->model),
@@ -709,6 +768,9 @@ downloader_view_add_download (DownloaderView *dv,
 #endif
 
 	dv->priv->source_id = g_timeout_add (100, (GSourceFunc) update_buttons_timeout_cb, dv);
+
+	/* see above */
+	g_object_unref (dv);
 }
 
 static void
@@ -988,6 +1050,7 @@ download_dialog_stop (DownloaderView *dv)
 	{
 		if (!l->data) continue;
 		webkit_download_cancel ((WebKitDownload*) l->data);
+		ephy_file_delete_uri (webkit_download_get_destination_uri ((WebKitDownload*) l->data));
 		g_object_unref (l->data);
 	}
 	
