@@ -509,7 +509,7 @@ sync_load_status (EphyWebView *view, GParamSpec *pspec, GtkWidget *proxy)
 	icon = GTK_WIDGET (g_object_get_data (G_OBJECT (proxy), "icon"));
 	g_return_if_fail (spinner != NULL && icon != NULL);
 
-	if (ephy_web_view_get_load_status (view))
+	if (ephy_web_view_is_loading (view))
 	{
 		gtk_widget_hide (icon);
 		gtk_widget_show (spinner);
@@ -734,11 +734,6 @@ ephy_notebook_add_tab (EphyNotebook *notebook,
 					     NULL,
 					     position);
 
-	/* FIXME gtk bug! */
-	/* FIXME: this should be fixed in gtk 2.12; check & remove this! */
-	/* The signal handler may have reordered the tabs */
-	position = gtk_notebook_page_num (gnotebook, GTK_WIDGET (embed));
-
 	if (jump_to)
 	{
 		gtk_notebook_set_current_page (gnotebook, position);
